# TODO: detect client disconnect and kill serverprocess request
# TODO: with forking have some shared memory store


# server:
# http://docs.python.org/library/simplexmlrpcserver.html
# http://www.doughellmann.com/PyMOTW/SimpleXMLRPCServer/

# client:
# http://docs.python.org/library/xmlrpclib.html
# http://www.doughellmann.com/PyMOTW/xmlrpclib/

# src : http://www.gossamer-threads.com/lists/python/python/110074
# Socket Disconnection :
# I assume here that you are talking TCP/IP. 
#
#    If you are waiting on a receive [ data = connection.recv() ] then you will 
#    have an empty data string returned when the client disconnects. 
#
#    If you are sending you will get an error telling you the link was shut down.

import SocketServer
from SimpleXMLRPCServer import SimpleXMLRPCServer,SimpleXMLRPCRequestHandler
import Cookie 
import time
import pprint
import tempfile

import compare
import sys


#server = SimpleXMLRPCServer(("localhost", 8000))
# -> does jobs sequential
# better is balancing jobs of cpu's using ForkingMixIn
#
# note: python can only run on thread at the time because 
# of global lock(GIL) because python is not threadsafe. Thus it should
# give the same performance as just sequential. However our job mainly consists
# of starting commandline tools such as caesor/svl which run
# on its own process! -> thus effectively doing multi cpu balancing
#
# Thus you can also use ThreadingMixIn which is cheaper
# in starting new jobs, because starting a new process is more
# costly then starting a new thread
# note: ThreadingMixIn  is also portable to other os's where 
#       ForkingMixIn is only supported on linux

# Threaded mix-in
class AsyncThreadXMLRPCServer(SocketServer.ThreadingMixIn,SimpleXMLRPCServer): pass


def compare_files_as_strings(str1,ext1,str2,ext2,min_range,max_range):
    #sys.stdout=sys.stderr # redirect stdout to stderr which is already unbuffered  (autflush)

    #sys.stdout=os.fdopen(sys.stdout.fileno(), 'w', 0) # make stdout unbuffered  : autoflush
    # `-> problem : second time called it will give bad file descripter error  ??  -> solution do redirect globally once!
    #   -> probably to do with smart catching exceptions by server when running this code

    print "----------> start compare" 
    import tempfile,shutil
    tmpdir=tempfile.mkdtemp(prefix="sut_compare_server_")
    filepath1=tmpdir + "/a" + ext1  
    filepath2=tmpdir + "/b" + ext2  
    print filepath1, filepath2
    open(filepath1,'w').write(str1)
    open(filepath2,'w').write(str2)
    output_str=compare.compare(filepath1,filepath2,min_range,max_range,False)
    shutil.rmtree(tmpdir) 

    #output_str=output_str.replace("<","&lt;")
    #output_str=output_str.replace(">","&gt;")
    #output_str=''.join(c for c in output_str if ord(c) >= 10)
    print output_str
    print "----------> finish compare" 
    return output_str



def dump(name,data):
      print("    " + name + ": {0}".format(pprint.pformat(data)))

import daemon,os

def main(args):
  import uppaal2bcg
  script_dir=os.path.abspath(os.path.dirname(sys.argv[0]))
  #TODO: change working din into tmp!!
  errfile=open('/tmp/tomte_stderr','w')
  outfile=open('/tmp/tomte_stdout','w')
  sys.stdout=os.fdopen(sys.stdout.fileno(), 'w', 0) # make stdout unbuffered  : autoflush

  # detach_process=None does :
  #   - fork twice and run as daemon
  #   - exception: if started by init then it skips the double fork because unneeded when started by daemon  
  #                unneeded because : processes start by init already have some required features of daemon which makes forks unnecesarry :
  #                                    do not have tty (stdin=stdout=stderr=/dev/null) and is child of init(pid=1)  
  #with daemon.DaemonContext(detach_process=None,working_directory=script_dir):
  with daemon.DaemonContext(detach_process=None,stdout=outfile,stderr=errfile,working_directory=script_dir):

    server = AsyncThreadXMLRPCServer(('', 8000), SimpleXMLRPCRequestHandler,allow_none=True)
    # Register a function under a different name
    server.register_function(compare_files_as_strings, 'tomte_compare')

    #server.register_multicall_functions()
    #server.register_introspection_functions()

    # Run the server's main loop
    server.serve_forever()
    #(modelfile1,modelfile2,range_min,range_max,keeptmp)=compare.args_parsing(args)
    #remotecompare(modelfile1,modelfile2,range_min,range_max)

if __name__ == "__main__":
    try:
        main(sys.argv)
    except KeyboardInterrupt as e:
        print >> sys.stderr, "\n\nExiting on user cancel."
        sys.exit(1)
