

import sys, re, pprint  # modules from standard lib (python 2.6 and later)
import yaml
import argparse
import tempfile
import shutil
import os


import uppaal2bcg

# load libraries from ./lib
import lib.util,lib.check,lib.run,lib.error

import util

# TODO: add other lts types with nice lts2lts script
import learnlib_dot2jtorx_aut,jtorx_aut2cadp_aut
#import lts2lts

def tomte_aut2cadp_aut(infile,outfile):
    """
      in parameters serialization:   '_'    -> ' !'
    """


    f=file(infile)
    lines=f.readlines()

    strings=[lines[0].strip()]

    transitions=lines[1:]

    regexpr_transition=re.compile(r'\s*\(\s*(\d*)\s*,\s*([^,]*)\s*,\s*(\d*)\s*\)')
    for transition in transitions:
        match=regexpr_transition.match(transition)
        if match:
            match=match.groups()
            #label=regexpr_tag.sub('',match[2])
            label=match[1].strip('"').replace('_',' !')

            strings.append("(" + match[0] + "," + '"' + label + '"' +  "," +  match[2] + ")")

    result="\n".join(strings)
    f=open(outfile,'w')
    f.write(result)
    f.close()

def cadp_aut2tomte_aut(infile,outfile):
    """
      in parameters serialization:  ' !'  -> '_'
    """


    f=file(infile)
    lines=f.readlines()

    strings=[lines[0].strip()]

    transitions=lines[1:]

    regexpr_transition=re.compile(r'\s*\(\s*(\d*)\s*,\s*([^,]*)\s*,\s*(\d*)\s*\)')
    for transition in transitions:
        match=regexpr_transition.match(transition)
        if match:
            match=match.groups()
            #label=regexpr_tag.sub('',match[2])
            label=match[1].strip('"').replace(' !','_')

            strings.append("(" + match[0] + "," + '"' + label + '"' +  "," +  match[2] + ")")

    result="\n".join(strings)
    f=open(outfile,'w')
    f.write(result)
    f.close()



def uppaal2aut(uppaalfile,tomte_aut_file,range_min,range_max,keeptmp):
     tmpdir=tempfile.mkdtemp(prefix="sut_uppaal2tomteaut")
     name="model"

     bcgfile=tmpdir + '/' + name + '.bcg'
     uppaal2bcg.uppaal2bcg(uppaalfile,bcgfile,range_min,range_max,keeptmp)

     cadp_aut_file=tmpdir + '/' + name + '.cadp.aut'
     lib.run.run_command([ 'bcg_io',  bcgfile,cadp_aut_file ],cwd=tmpdir)

     cadp_aut2tomte_aut(cadp_aut_file, tomte_aut_file)


     if not keeptmp:
        shutil.rmtree(tmpdir)




#-------------------------------------------------------------------------------
#   main + input parameters parsing
#-------------------------------------------------------------------------------

def args_parsing(args):

    #parser = argparse.ArgumentParser(prog=os.path.basename(sys.argv[0]),epilog=get_models_help(models))  # prints epilog always on single line
    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    argparser.add_argument('modelfile', help='input uppaal model file')
    argparser.add_argument('autfile', help='aut output file')
    argparser.add_argument('range_min', help='minimum value of range used for input params when flattening the model')
    argparser.add_argument('range_max', help='maximum value of range used for input params when flattening the model')

    str=argparser.format_help()


    def myerror( message):
            print >>sys.stderr, ('%s: error: %s\n') % (argparser.prog, message)
            argparser.print_usage(sys.stderr)
            exit(1)

    argparser.error=myerror


    parse_result=argparser.parse_args(args[1:])

    # handle modelfile argument
    model_file=parse_result.modelfile
    # always use unix slashes in python!!
    # automatic convert any windows slashes
    # TODO: document this convention in python/txtdocs/
    model_file.replace('\\','/')
    util.check_isreadablefile(model_file)


    # handle output directory argument
    # - first get absolute path
    aut_file=parse_result.autfile
    aut_file=os.path.abspath(aut_file)   # on windows also interprets / as dir sep, and returns path with windows \
    aut_file.replace('\\','/')



    range_min=parse_result.range_min
    range_max=parse_result.range_max

    return  [model_file,aut_file,range_min,range_max]




def main(args):

    # parse arguments
    (uppaalfile,autfile,range_min,range_max)=args_parsing(args)

    # convert uppaal model 2 aut
    uppaal2aut(uppaalfile,autfile,range_min,range_max,True)



if __name__ == "__main__":
    main(sys.argv)

